/****************************************************************************
** Meta object code from reading C++ file 'JavascriptInvocation.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../JavascriptInvocation.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'JavascriptInvocation.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_JavascriptInvocation_t {
    QByteArrayData data[32];
    char stringdata0[290];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_JavascriptInvocation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_JavascriptInvocation_t qt_meta_stringdata_JavascriptInvocation = {
    {
QT_MOC_LITERAL(0, 0, 20), // "JavascriptInvocation"
QT_MOC_LITERAL(1, 21, 9), // "leftClick"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 1), // "x"
QT_MOC_LITERAL(4, 34, 1), // "y"
QT_MOC_LITERAL(5, 36, 4), // "keys"
QT_MOC_LITERAL(6, 41, 10), // "rightClick"
QT_MOC_LITERAL(7, 52, 11), // "doubleClick"
QT_MOC_LITERAL(8, 64, 9), // "clickTest"
QT_MOC_LITERAL(9, 74, 11), // "QWebElement"
QT_MOC_LITERAL(10, 86, 7), // "element"
QT_MOC_LITERAL(11, 94, 9), // "absoluteX"
QT_MOC_LITERAL(12, 104, 9), // "absoluteY"
QT_MOC_LITERAL(13, 114, 13), // "clickPosition"
QT_MOC_LITERAL(14, 128, 4), // "left"
QT_MOC_LITERAL(15, 133, 3), // "top"
QT_MOC_LITERAL(16, 137, 5), // "width"
QT_MOC_LITERAL(17, 143, 6), // "height"
QT_MOC_LITERAL(18, 150, 5), // "hover"
QT_MOC_LITERAL(19, 156, 8), // "keypress"
QT_MOC_LITERAL(20, 165, 12), // "namedKeydown"
QT_MOC_LITERAL(21, 178, 7), // "keyName"
QT_MOC_LITERAL(22, 186, 10), // "namedKeyup"
QT_MOC_LITERAL(23, 197, 13), // "namedKeypress"
QT_MOC_LITERAL(24, 211, 9), // "modifiers"
QT_MOC_LITERAL(25, 221, 6), // "render"
QT_MOC_LITERAL(26, 228, 12), // "functionName"
QT_MOC_LITERAL(27, 241, 15), // "allowUnattached"
QT_MOC_LITERAL(28, 257, 9), // "arguments"
QT_MOC_LITERAL(29, 267, 5), // "error"
QT_MOC_LITERAL(30, 273, 8), // "key_enum"
QT_MOC_LITERAL(31, 282, 7) // "Qt::Key"

    },
    "JavascriptInvocation\0leftClick\0\0x\0y\0"
    "keys\0rightClick\0doubleClick\0clickTest\0"
    "QWebElement\0element\0absoluteX\0absoluteY\0"
    "clickPosition\0left\0top\0width\0height\0"
    "hover\0keypress\0namedKeydown\0keyName\0"
    "namedKeyup\0namedKeypress\0modifiers\0"
    "render\0functionName\0allowUnattached\0"
    "arguments\0error\0key_enum\0Qt::Key"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_JavascriptInvocation[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       5,  128, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    3,   69,    2, 0x02 /* Public */,
       6,    3,   76,    2, 0x02 /* Public */,
       7,    3,   83,    2, 0x02 /* Public */,
       8,    3,   90,    2, 0x02 /* Public */,
      13,    5,   97,    2, 0x02 /* Public */,
      18,    2,  108,    2, 0x02 /* Public */,
      19,    1,  113,    2, 0x02 /* Public */,
      20,    1,  116,    2, 0x02 /* Public */,
      22,    1,  119,    2, 0x02 /* Public */,
      23,    2,  122,    2, 0x02 /* Public */,
      25,    0,  127,    2, 0x02 /* Public */,

 // methods: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QVariantList,    3,    4,    5,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QVariantList,    3,    4,    5,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QVariantList,    3,    4,    5,
    QMetaType::Bool, 0x80000000 | 9, QMetaType::Int, QMetaType::Int,   10,   11,   12,
    QMetaType::QVariantMap, 0x80000000 | 9, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,   10,   14,   15,   16,   17,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   11,   12,
    QMetaType::Void, QMetaType::QChar,    2,
    QMetaType::Void, QMetaType::QString,   21,
    QMetaType::Void, QMetaType::QString,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   21,   24,
    QMetaType::QString,

 // properties: name, type, flags
      26, QMetaType::QString, 0x00095001,
      27, QMetaType::Bool, 0x00095001,
      28, QMetaType::QStringList, 0x00095001,
      29, QMetaType::QVariant, 0x00095103,
      30, 0x80000000 | 31, 0x00095008,

       0        // eod
};

void JavascriptInvocation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<JavascriptInvocation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->leftClick((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< QVariantList(*)>(_a[3]))); break;
        case 1: _t->rightClick((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< QVariantList(*)>(_a[3]))); break;
        case 2: _t->doubleClick((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< QVariantList(*)>(_a[3]))); break;
        case 3: { bool _r = _t->clickTest((*reinterpret_cast< QWebElement(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { QVariantMap _r = _t->clickPosition((*reinterpret_cast< QWebElement(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->hover((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->keypress((*reinterpret_cast< QChar(*)>(_a[1]))); break;
        case 7: _t->namedKeydown((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 8: _t->namedKeyup((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 9: _t->namedKeypress((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 10: { QString _r = _t->render();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWebElement >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWebElement >(); break;
            }
            break;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<JavascriptInvocation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->functionName(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->allowUnattached(); break;
        case 2: *reinterpret_cast< QStringList*>(_v) = _t->arguments(); break;
        case 3: *reinterpret_cast< QVariant*>(_v) = _t->getError(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<JavascriptInvocation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setError(*reinterpret_cast< QVariant*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject JavascriptInvocation::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_JavascriptInvocation.data,
    qt_meta_data_JavascriptInvocation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *JavascriptInvocation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JavascriptInvocation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_JavascriptInvocation.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int JavascriptInvocation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
